/*
    This file is part of KOrganizer.

    Copyright (c) 2003 Cornelius Schumacher <schumacher@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

    As a special exception, permission is given to link this program
    with any edition of TQt, and distribute the resulting executable,
    without including the source code for TQt in the source distribution.
*/
#ifndef NAVIGATORBAR_H
#define NAVIGATORBAR_H

#include <libkcal/incidencebase.h>

#include <tqlabel.h>

class TQPushButton;
class TQFrame;

class ActiveLabel : public TQLabel
{
    TQ_OBJECT
  
  public:
    ActiveLabel( TQWidget *parent, const char *name = 0 );

  signals:
    void clicked();

  protected:
    void mouseReleaseEvent ( TQMouseEvent * e );
};


class NavigatorBar: public TQWidget
{
    TQ_OBJECT
  
  public:
    NavigatorBar( TQWidget *parent = 0, const char *name = 0 );
    ~NavigatorBar();

    void showButtons( bool left, bool right );

  public slots:
    void selectDates( const KCal::DateList & );

  signals:
    void nextMonthClicked();
    void prevMonthClicked();
    void nextYearClicked();
    void prevYearClicked();
    void monthSelected( int month );
    void yearSelected( int year );

  private slots:
    void selectMonthFromMenu();
    void selectYearFromMenu();

  private:
    bool mHasMinWidth;

    TQDate mDate;

    TQPushButton *mPrevYear;
    TQPushButton *mPrevMonth;
    ActiveLabel *mMonth;
    ActiveLabel *mYear;
    TQPushButton *mNextMonth;
    TQPushButton *mNextYear;
};

#endif
